<?php

namespace WPDM\__;

class AI {

	static function callGemini( string $prompt ): string|null {
		$apiKey = get_option( '__wpdm_gemini_api_key' );
		if ( ! $apiKey ) {
			return 'Gemini API Key not set';
		}

		$url = 'https://generativelanguage.googleapis.com/v1beta/models/gemini-pro:generateContent';

		// Prepare the request data
		$data = [
			'contents' => [
				[
					'parts' => [
						[ 'text' => $prompt . ". Give me the exact result for the query, no extra explanation or anything else." ]
					]
				],
			],
		];


		// Merge user-provided options (if any) with defaults
		$requestOptions = [
			'temperature' => 0.8,
			'top_p'       => 1,
			'top_k'       => 40,
		];

		$data['generationConfig'] = $requestOptions;

		$headers = [
			'Content-Type: application/json',
			'x-goog-api-key: ' . $apiKey
		];

		$ch = curl_init( $url );
		curl_setopt( $ch, CURLOPT_POST, 1 );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode( $data ) );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, $headers );

		$response = curl_exec( $ch );

		if ( curl_errno( $ch ) ) {
			// Handle cURL errors
			error_log( "cURL Error: " . curl_error( $ch ) );
			curl_close( $ch );

			return null;
		}

		curl_close( $ch );

		$decodedResponse = json_decode( $response, true );

		if ( isset( $decodedResponse['candidates'] ) && ! empty( $decodedResponse['candidates'] ) ) {
			return $decodedResponse['candidates'][0]['content']['parts'][0]['text'];
		} else {
			error_log( "Gemini API Error: Response doesn't contain the text content." . json_encode( $decodedResponse ) );

			return null;
		}

	}

	static function callOpenAI( $prompt ) {
		// Your OpenAI API key
		$apiKey = get_option('__wpdm_openai_api_key');

		// OpenAI ChatGPT API endpoint
		$apiUrl = 'https://api.openai.com/v1/chat/completions';

		// Prepare the data payload
		$data = [
			'model'       => 'gpt-4', // Specify the GPT-4 model
			'messages'    => [
				[ 'role' => 'system', 'content' => 'You are a helpful assistant.' ],
				[ 'role' => 'user', 'content' => $prompt . ". Give me the exact result for the query, no extra explanation or anything else." ],
			],
			'max_tokens'  => 1000,
			'temperature' => 0.7,
		];

		// Initialize cURL session
		$ch = curl_init();

		// Set cURL options
		curl_setopt( $ch, CURLOPT_URL, $apiUrl );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_POST, true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode( $data ) );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Authorization: Bearer ' . $apiKey,
		] );

		// Execute the cURL request
		$response = curl_exec( $ch );

		// Check for cURL errors
		if ( curl_errno( $ch ) ) {
			throw new \Exception( 'Request Error: ' . curl_error( $ch ) );
		}

		// Close the cURL session
		curl_close( $ch );

		// Decode and return the response
		$response = json_decode( $response, true );

		return $response['choices'][0]['message']['content'];
	}

	static function callAI( string $prompt ) {
		$ai = get_option('__wpdm_ai', 'gemini');
		if($ai !== 'openai')
			return self::callGemini($prompt);
		return self::callOpenAI($prompt);
	}


}
